Summary
Callbacks
Return the text that will be placed into a migration as an attribute default.
Callbacks
@callback migration_type(Ash.Type.constraints()) :: term()
@callback postgres_reference_expr(Ash.Type.t(), Ash.Type.constraints(), term()) :: {:ok, term()} | :error
@callback value_to_postgres_default(Ash.Type.t(), Ash.Type.constraints(), term()) :: {:ok, String.t()} | :error
Return the text that will be placed into a migration as an attribute default.
The returned string is placed verbatim into generated migrations.
If this callback is not implemented, the migration generator falls back to
EctoMigrationDefault.